from nonebot.adapters.qq import Bot
from nonebot.adapters.qq.event import (
    C2CMessageCreateEvent,
    ChannelEvent,
    DirectMessageCreateEvent,
    DirectMessageDeleteEvent,
    Event,
    GroupAtMessageCreateEvent,
    GuildEvent,
    GuildMemberEvent,
    GuildMessageEvent,
    InteractionCreateEvent,
    MessageDeleteEvent,
)
from nonebot.exception import ActionFailed

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Role, Scene, SceneType, User

ROLES = {
    "4": ("OWNER", 100, "创建者"),
    "2": ("ADMINISTRATOR", 10, "管理员"),
    "5": ("CHANNEL_ADMINISTRATOR", 8, "子频道管理员"),
    "1": ("MEMBER", 1, "成员"),
}


CHANNEL_TYPE = {
    -1: SceneType.PRIVATE,
    0: SceneType.CHANNEL_TEXT,
    2: SceneType.CHANNEL_VOICE,
    4: SceneType.CHANNEL_CATEGORY,
}


class InfoFetcher(BaseInfoFetcher):
    def get_session_id(self, event: Event) -> str:
        if isinstance(event, MessageDeleteEvent):
            return f"{event.get_session_id()}_{event.op_user.id}"
        if isinstance(event, GuildEvent):
            return f"guild_{event.id}_{event.op_user_id}"
        if isinstance(event, GuildMemberEvent):
            return f"{event.get_session_id()}_{event.op_user_id}"
        if isinstance(event, ChannelEvent):
            return f"channel_{event.guild_id}_{event.id}_{event.op_user_id}"
        return event.get_session_id()

    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            avatar=data["avatar"],
        )

    def extract_scene(self, data):
        if "group_id" in data:
            return Scene(
                id=data["group_id"],
                type=SceneType.GROUP,
            )
        if "guild_id" in data:
            if "channel_id" in data:
                return Scene(
                    id=data["channel_id"],
                    name=data.get("channel_name"),
                    type=CHANNEL_TYPE.get(data.get("channel_type", 0), SceneType.CHANNEL_TEXT),
                    parent=Scene(
                        id=data["guild_id"],
                        name=data.get("guild_name"),
                        type=SceneType.GUILD,
                        avatar=data.get("guild_avatar"),
                    ),
                )
            return Scene(
                id=data["guild_id"],
                name=data.get("guild_name"),
                type=SceneType.GUILD,
                avatar=data.get("guild_avatar"),
            )
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
            name=data["name"],
            avatar=data["avatar"],
        )

    def extract_member(self, data, user: User | None):
        if "group_id" in data:
            if user:
                return Member(user, nick=data["nickname"])
            return Member(
                User(
                    id=data["user_id"],
                    name=data["name"],
                    avatar=data.get("avatar"),
                ),
                nick=data["nickname"],
            )
        if "guild_id" in data:
            if user:
                return Member(user, nick=data["nickname"], roles=data.get("roles", []), joined_at=data.get("joined_at"))
            return Member(
                User(
                    id=data["user_id"],
                    name=data["name"],
                    avatar=data.get("avatar"),
                ),
                nick=data["nickname"],
                roles=data.get("roles", []),
                joined_at=data.get("joined_at"),
            )
        return None

    async def query_user(self, bot: Bot, user_id: str):
        if user_id == bot.self_id:
            info = await bot.me()
            return User(
                id=info.id,
                name=info.username,
                avatar=info.avatar,
            )
        return User(
            id=user_id,
            name="",
            avatar=f"https://q.qlogo.cn/qqapp/{bot.bot_info.id}/{user_id}/100",
        )

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.GUILD:
            guild = await bot.get_guild(guild_id=scene_id)
            return Scene(id=guild.id, type=SceneType.GUILD, name=guild.name, avatar=guild.icon)

        elif scene_type >= SceneType.CHANNEL_TEXT:
            channel = await bot.get_channel(channel_id=scene_id)
            guild = await bot.get_guild(guild_id=channel.guild_id)
            return Scene(
                id=channel.id,
                type=CHANNEL_TYPE.get(channel.type, SceneType.CHANNEL_TEXT),
                name=channel.name,
                parent=Scene(id=guild.id, type=SceneType.GUILD, name=guild.name, avatar=guild.icon),
            )

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type >= SceneType.GUILD:
            member = await bot.get_member(guild_id=parent_scene_id, user_id=user_id)
            return Member(
                User(
                    id=member.user.id if member.user else user_id,
                    name=(member.user.username if member.user else "") or "",
                    avatar=member.user.avatar if member.user else None,
                ),
                nick=member.nick,
                roles=await _handle_roles(bot, parent_scene_id, None, member.roles or []),
                joined_at=member.joined_at,
            )
        if scene_type == SceneType.GROUP:
            return Member(
                User(
                    id=user_id,
                    name="",
                    avatar=f"https://q.qlogo.cn/qqapp/{bot.bot_info.id}/{user_id}/100",
                ),
                nick="",
            )

    def query_users(self, bot: Bot):
        raise NotImplementedError

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is not None and scene_type < SceneType.GUILD:
            return

        guilds = await bot.guilds(limit=100)
        while guilds:
            for guild in guilds:
                if parent_scene_id is None or guild.id == parent_scene_id:
                    _guild = Scene(id=guild.id, type=SceneType.GUILD, name=guild.name, avatar=guild.icon)
                    if scene_type is None or scene_type == SceneType.GUILD:
                        yield _guild
                    if scene_type == SceneType.GUILD:
                        continue
                    channels = await bot.get_channels(guild_id=guild.id)
                    for channel in channels:
                        yield Scene(
                            id=channel.id,
                            type=CHANNEL_TYPE.get(channel.type, SceneType.CHANNEL_TEXT),
                            name=channel.name,
                            parent=_guild,
                        )
            if len(guilds) < 100:
                break
            guilds = await bot.guilds(limit=100, after=guilds[-1].id)

    def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        raise NotImplementedError

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.qq,
            "scope": SupportScope.qq_api,
        }


fetcher = InfoFetcher(SupportAdapter.qq)


async def _handle_roles(bot: Bot, guild_id: str, channel_id: str | None, roles: list[str]):
    if not roles:
        return [Role(*ROLES["1"])]
    res = []
    try:
        resp = await bot.get_guild_roles(guild_id=guild_id)
        roles_info = {r.id: r.name for r in resp.roles}
    except ActionFailed:
        roles_info = {}
    for role in roles:
        if role in ROLES:
            res.append(ROLES[role])
            continue
        if not channel_id:
            res.append(("MEMBER", 1, roles_info.get(role, "成员")))
            continue
        try:
            perm = await bot.get_channel_roles_permissions(channel_id=channel_id, role_id=role)
            if perm.permissions & 0b10 == 0b10:
                res.append(("MEMBER", perm.permissions, roles_info.get(role, "成员")))
            else:
                res.append(("CHANNEL_ADMINISTRATOR", perm.permissions, roles_info.get(role, "子频道管理员")))
        except ActionFailed:
            res.append(("MEMBER", 1, roles_info.get(role, "成员")))
    if not res:
        return [Role(*ROLES["1"])]
    return [Role(*r) for r in res]


@fetcher.supply
async def _(bot: Bot, event: InteractionCreateEvent):
    if event.chat_type == 2:
        return {
            "user_id": event.user_openid,
            "name": "",
            "nickname": "",
            "avatar": f"https://q.qlogo.cn/qqapp/{bot.bot_info.id}/{event.user_openid}/100",
        }
    if event.chat_type == 1:
        return {
            "user_id": event.group_member_openid,
            "name": "",
            "nickname": "",
            "avatar": f"https://q.qlogo.cn/qqapp/{bot.bot_info.id}/{event.group_member_openid}/100",
            "group_id": event.group_openid,
        }
    base = {
        "user_id": event.data.resolved.user_id,
        "name": "",
        "nickname": "",
        "avatar": None,
        "guild_id": event.guild_id,
        "channel_id": event.channel_id,
    }
    try:
        member = await bot.get_member(guild_id=event.guild_id, user_id=event.data.resolved.user_id)  # type: ignore
        base["name"] = (member.user.username if member.user else "") or ""
        base["nickname"] = member.nick or ""
        base["roles"] = await _handle_roles(bot, event.guild_id, event.channel_id, member.roles or [])  # type: ignore
        base["joined_at"] = member.joined_at
    except ActionFailed:
        pass
    try:
        guild = await bot.get_guild(guild_id=event.guild_id)  # type: ignore
        base["guild_name"] = guild.name
        base["guild_avatar"] = guild.icon
        channel = await bot.get_channel(channel_id=event.channel_id)  # type: ignore
        base["channel_name"] = channel.name
        base["channel_type"] = channel.type
    except ActionFailed:
        pass
    return base


@fetcher.supply
async def _(bot: Bot, event: C2CMessageCreateEvent):
    return {
        "user_id": event.author.user_openid,
        "name": "",
        "nickname": "",
        "avatar": f"https://q.qlogo.cn/qqapp/{bot.bot_info.id}/{event.author.user_openid}/100",
    }


@fetcher.supply
async def _(bot: Bot, event: GroupAtMessageCreateEvent):
    return {
        "user_id": event.author.member_openid,
        "name": "",
        "nickname": "",
        "avatar": f"https://q.qlogo.cn/qqapp/{bot.bot_info.id}/{event.author.member_openid}/100",
        "group_id": event.group_openid,
    }


@fetcher.supply_wildcard
async def _(bot: Bot, event: Event):
    if isinstance(event, GuildMessageEvent):
        base: dict = {
            "user_id": event.author.id,
            "name": event.author.username or "",
            "nickname": "",
            "avatar": event.author.avatar,
            "guild_id": event.guild_id,
            "channel_id": event.channel_id,
        }
        if event.member:
            base |= {
                "nickname": event.member.nick or "",
                "roles": await _handle_roles(bot, event.guild_id, event.channel_id, event.member.roles or []),
                "joined_at": event.member.joined_at,
            }
        try:
            guild = await bot.get_guild(guild_id=event.guild_id)
            base["guild_name"] = guild.name
            base["guild_avatar"] = guild.icon
            channel = await bot.get_channel(channel_id=event.channel_id)
            base["channel_name"] = channel.name
            base["channel_type"] = -1 if isinstance(event, DirectMessageCreateEvent) else channel.type
        except ActionFailed:
            pass
        return base
    if isinstance(event, MessageDeleteEvent):
        message_event = event.message
        base = {
            "user_id": message_event.author.id,
            "name": message_event.author.username or "",
            "nickname": "",
            "avatar": message_event.author.avatar,
            "guild_id": message_event.guild_id,
            "channel_id": message_event.channel_id,
        }
        if message_event.member:
            base |= {
                "nickname": message_event.member.nick or "",
                "roles": await _handle_roles(
                    bot, message_event.guild_id, message_event.channel_id, message_event.member.roles or []
                ),
                "joined_at": message_event.member.joined_at,
            }
        try:
            guild = await bot.get_guild(guild_id=message_event.guild_id)
            base["guild_name"] = guild.name
            base["guild_avatar"] = guild.icon
            channel = await bot.get_channel(channel_id=message_event.channel_id)
            base["channel_name"] = channel.name
            base["channel_type"] = -1 if isinstance(event, DirectMessageDeleteEvent) else channel.type
        except ActionFailed:
            pass
        base["operator"] = {
            "user_id": event.op_user.id,
            "name": event.op_user.username or "",
            "nickname": "",
            "avatar": event.op_user.avatar,
        }
        try:
            operator = await bot.get_member(guild_id=message_event.guild_id, user_id=event.op_user.id)
            base["operator"] |= {
                "nickname": operator.nick or "",
                "roles": await _handle_roles(
                    bot, message_event.guild_id, message_event.channel_id, operator.roles or []
                ),
                "joined_at": operator.joined_at,
            }
        except ActionFailed:
            pass
        return base
    if isinstance(event, GuildEvent):
        me = bot.self_info
        base = {
            "user_id": me.id,
            "name": me.username or "",
            "nickname": "",
            "avatar": me.avatar,
            "guild_id": event.id,
            "guild_name": event.name,
            "guild_avatar": event.icon,
        }
        try:
            operator = await bot.get_member(guild_id=event.id, user_id=event.op_user_id)
            base["operator"] = {
                "user_id": event.op_user_id,
                "name": (operator.user.username if operator.user else "") or "",
                "nickname": operator.nick or "",
                "avatar": operator.user.avatar if operator.user else None,
                "roles": await _handle_roles(bot, event.id, None, operator.roles or []),
                "joined_at": operator.joined_at,
            }
        except ActionFailed:
            pass
        return base
    if isinstance(event, GuildMemberEvent):
        base = {
            "user_id": event.user.id,  # type: ignore
            "name": (event.user.username if event.user else "") or "",
            "nickname": event.nick or "",
            "avatar": event.user.avatar if event.user else None,
            "guild_id": event.guild_id,
            "roles": await _handle_roles(bot, event.guild_id, None, event.roles or []),
            "joined_at": event.joined_at,
        }
        try:
            guild = await bot.get_guild(guild_id=event.guild_id)
            base["guild_name"] = guild.name
            base["guild_avatar"] = guild.icon
        except ActionFailed:
            pass
        try:
            operator = await bot.get_member(guild_id=event.guild_id, user_id=event.op_user_id)
            base["operator"] = {
                "user_id": event.op_user_id,
                "name": (operator.user.username if operator.user else "") or "",
                "nickname": operator.nick or "",
                "avatar": operator.user.avatar if operator.user else None,
                "roles": await _handle_roles(bot, event.guild_id, None, operator.roles or []),
                "joined_at": operator.joined_at,
            }
        except ActionFailed:
            pass
        return base
    if isinstance(event, ChannelEvent):
        me = bot.self_info
        base = {
            "user_id": me.id,
            "name": me.username or "",
            "nickname": "",
            "avatar": me.avatar,
            "guild_id": event.guild_id,
            "channel_id": event.id,
            "channel_name": event.name,
            "channel_type": event.type,
        }
        try:
            guild = await bot.get_guild(guild_id=event.guild_id)
            base["guild_name"] = guild.name
            base["guild_avatar"] = guild.icon
        except ActionFailed:
            pass
        try:
            operator = await bot.get_member(guild_id=event.guild_id, user_id=event.op_user_id)
            base["operator"] = {
                "user_id": event.op_user_id,
                "name": (operator.user.username if operator.user else "") or "",
                "nickname": operator.nick or "",
                "avatar": operator.user.avatar if operator.user else None,
                "roles": await _handle_roles(bot, event.guild_id, event.id, operator.roles or []),
                "joined_at": operator.joined_at,
            }
        except ActionFailed:
            pass
        return base
    raise NotImplementedError



from datetime import datetime, timedelta
import json


class DatetimeJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return int(obj.timestamp())
        elif isinstance(obj, timedelta):
            return obj.total_seconds()
        return json.JSONEncoder.default(self, obj)


if __name__ == "__main__":
    data = {
        "1": datetime.now(),
        "2": timedelta(days=1),
        "3": {"4": datetime.now(), "5": timedelta(days=1)},
        "4": [datetime.now(), timedelta(days=1)],
    }
    print(json.dumps(data, cls=DatetimeJsonEncoder))



from nonebot_plugin_uninfo.constraint import SupportAdapter
from nonebot_plugin_uninfo.fetch import InfoFetcher
from nonebot_plugin_uninfo.loader import BaseLoader


class Loader(BaseLoader):
    def get_adapter(self) -> SupportAdapter:
        return SupportAdapter.qq

    def get_fetcher(self) -> InfoFetcher:
        from .main import fetcher

        return fetcher



from datetime import timedelta

from nonebot.adapters.onebot.v12 import Bot
from nonebot.adapters.onebot.v12.event import (
    ChannelCreateEvent,
    ChannelDeleteEvent,
    ChannelMemberDecreaseEvent,
    ChannelMemberIncreaseEvent,
    ChannelMessageDeleteEvent,
    ChannelMessageEvent,
    FriendDecreaseEvent,
    FriendIncreaseEvent,
    GroupMemberDecreaseEvent,
    GroupMemberIncreaseEvent,
    GroupMessageDeleteEvent,
    GroupMessageEvent,
    GuildMemberDecreaseEvent,
    GuildMemberIncreaseEvent,
    PrivateMessageDeleteEvent,
    PrivateMessageEvent,
)
from nonebot.exception import ActionFailed
from nonebot.internal.adapter import Event

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, MuteInfo, Scene, SceneType, User


class InfoFetcher(BaseInfoFetcher):
    def get_session_id(self, event: Event) -> str:
        if isinstance(event, (GuildMemberDecreaseEvent, GuildMemberIncreaseEvent)):
            return f"{event.get_session_id()}_{event.operator_id}"
        return event.get_session_id()

    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            nick=data["nickname"],
        )

    def extract_scene(self, data):
        if "group_id" in data:
            return Scene(
                id=data["group_id"],
                type=SceneType.GROUP,
                name=data.get("group_name"),
            )
        if "guild_id" in data:
            if "channel_id" in data:
                return Scene(
                    id=data["channel_id"],
                    type=SceneType.CHANNEL_TEXT,
                    name=data["channel_name"],
                    parent=Scene(
                        id=data["guild_id"],
                        type=SceneType.GUILD,
                        name=data.get("guild_name"),
                    ),
                )
            return Scene(
                id=data["guild_id"],
                type=SceneType.GUILD,
                name=data.get("guild_name"),
            )
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
        )

    def extract_member(self, data, user: User | None):
        if "group_id" not in data:
            return None
        if "guild_id" not in data:
            return None
        if user:
            return Member(
                user=user,
                nick=data["displayname"],
                mute=(
                    MuteInfo(muted=True, duration=timedelta(seconds=data["mute_duration"]))
                    if "mute_duration" in data
                    else None
                ),
            )
        return Member(
            User(
                id=data["user_id"],
                name=data["name"],
                nick=data.get("nickname"),
            ),
            nick=data["displayname"],
            mute=(
                MuteInfo(muted=True, duration=timedelta(seconds=data["mute_duration"]))
                if "mute_duration" in data
                else None
            ),
        )

    async def query_user(self, bot: Bot, user_id: str):
        if user_id == str(bot.self_id):
            info = await bot.get_self_info()
        else:
            info = await bot.get_user_info(user_id=user_id)
        data = {
            "user_id": info["user_id"],
            "name": info["user_name"],
            "nickname": info.get("user_remark", info["user_displayname"]),
        }
        return self.extract_user(data)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            if user := await self.query_user(bot, scene_id):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "nickname": user.nick,
                }
                return self.extract_scene(data)

        elif scene_type == SceneType.GROUP:
            group_info = await bot.get_group_info(group_id=scene_id)
            data = {
                "group_id": group_info["group_id"],
                "group_name": group_info["group_name"],
            }
            return self.extract_scene(data)

        elif scene_type == SceneType.GUILD:
            guild_info = await bot.get_guild_info(guild_id=scene_id)
            data = {
                "guild_id": guild_info["guild_id"],
                "guild_name": guild_info["guild_name"],
            }
            return self.extract_scene(data)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type == SceneType.GROUP:
            group_id = parent_scene_id
            member_info = await bot.get_group_member_info(group_id=group_id, user_id=user_id)
            data = {
                "group_id": group_id,
                "user_id": member_info["user_id"],
                "name": member_info["user_name"],
                "displayname": member_info["user_displayname"],
            }
            return self.extract_member(data, None)

        elif scene_type == SceneType.GUILD:
            guild_id = parent_scene_id
            member_info = await bot.get_guild_member_info(guild_id=guild_id, user_id=user_id)
            data = {
                "guild_id": guild_id,
                "user_id": member_info["user_id"],
                "name": member_info["user_name"],
                "displayname": member_info["user_displayname"],
            }
            return self.extract_member(data, None)

    async def query_users(self, bot: Bot):
        friends = await bot.get_friend_list()
        for friend in friends:
            data = {
                "user_id": friend["user_id"],
                "name": friend["user_name"],
                "nickname": friend["user_remark"],
            }
            yield self.extract_user(data)

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is None or scene_type == SceneType.PRIVATE:
            async for user in self.query_users(bot):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "avatar": user.avatar,
                }
                yield self.extract_scene(data)

        if scene_type is None or scene_type == SceneType.GROUP:
            groups = await bot.get_group_list()
            for group in groups:
                data = {
                    "group_id": group["group_id"],
                    "group_name": group["group_name"],
                }
                yield self.extract_scene(data)

        if scene_type is None or scene_type >= SceneType.GUILD:
            guilds = await bot.get_guild_list()
            for guild in guilds:
                if parent_scene_id is not None and guild["guild_id"] != parent_scene_id:
                    continue
                data = {
                    "guild_id": guild["guild_id"],
                    "guild_name": guild["guild_name"],
                }
                if scene_type is None or scene_type == SceneType.GUILD:
                    yield self.extract_scene(data)
                if scene_type == SceneType.GUILD:
                    continue
                channels = await bot.get_channel_list(guild_id=guild["guild_id"])
                for channel in channels:
                    data = {
                        "guild_id": guild["guild_id"],
                        "guild_name": guild["guild_name"],
                        "channel_id": channel["channel_id"],
                        "channel_name": channel["channel_name"],
                    }
                    yield self.extract_scene(data)

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        if scene_type == SceneType.GROUP:
            group_id = parent_scene_id
            group_members = await bot.get_group_member_list(group_id=group_id)
            for member in group_members:
                data = {
                    "group_id": group_id,
                    "user_id": member["user_id"],
                    "name": member["user_name"],
                    "displayname": member["user_displayname"],
                }
                yield self.extract_member(data, None)

        elif scene_type == SceneType.GUILD:
            guild_id = parent_scene_id
            guild_members = await bot.get_guild_member_list(guild_id=guild_id)
            for member in guild_members:
                data = {
                    "guild_id": guild_id,
                    "user_id": member["user_id"],
                    "name": member["user_name"],
                    "displayname": member["user_displayname"],
                }
                yield self.extract_member(data, None)

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.onebot12,
            "scope": SupportScope.ensure_ob12(bot.platform),
        }


fetcher = InfoFetcher(SupportAdapter.onebot12)


@fetcher.supply
async def _(
    bot: Bot,
    event: PrivateMessageDeleteEvent | PrivateMessageEvent | FriendDecreaseEvent | FriendIncreaseEvent,
):
    try:
        user_info = await bot.get_user_info(user_id=event.user_id)
    except ActionFailed:
        user_info = {}
    return {
        "user_id": event.user_id,
        "name": user_info.get("user_name"),
        "nickname": user_info.get("user_remark"),
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: GroupMemberDecreaseEvent | GroupMemberIncreaseEvent | GroupMessageDeleteEvent | GroupMessageEvent,
):
    try:
        group_info = await bot.get_group_info(group_id=event.group_id)
    except ActionFailed:
        group_info = {}
    try:
        user_info = await bot.get_user_info(user_id=event.user_id)
    except ActionFailed:
        user_info = {}
    try:
        member_info = await bot.get_group_member_info(
            group_id=event.group_id,
            user_id=event.user_id,
        )
    except ActionFailed:
        member_info = {}
    return {
        "group_id": event.group_id,
        "group_name": group_info.get("group_name"),
        "user_id": event.user_id,
        "name": user_info.get("user_name"),
        "nickname": user_info.get("user_remark"),
        "displayname": member_info.get("user_displayname"),
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: ChannelMemberDecreaseEvent | ChannelMemberIncreaseEvent | ChannelMessageDeleteEvent | ChannelMessageEvent,
):
    try:
        guild_info = await bot.get_group_info(group_id=event.guild_id)
    except ActionFailed:
        guild_info = {}
    try:
        channel_info = await bot.get_channel_info(guild_id=event.guild_id, channel_id=event.channel_id)
    except ActionFailed:
        channel_info = {}
    try:
        user_info = await bot.get_guild_member_info(guild_id=event.guild_id, user_id=event.user_id)
    except ActionFailed:
        user_info = {}
    try:
        member_info = await bot.get_channel_member_info(
            guild_id=event.guild_id,
            channel_id=event.channel_id,
            user_id=event.user_id,
        )
    except ActionFailed:
        member_info = {}
    return {
        "guild_id": event.guild_id,
        "guild_name": guild_info.get("guild_name"),
        "channel_id": event.channel_id,
        "channel_name": channel_info.get("channel_name"),
        "user_id": event.user_id,
        "name": user_info.get("user_name"),
        "nickname": user_info.get("user_displayname"),
        "displayname": member_info.get("user_displayname"),
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: ChannelCreateEvent | ChannelDeleteEvent,
):
    try:
        guild_info = await bot.get_group_info(group_id=event.guild_id)
    except ActionFailed:
        guild_info = {}
    try:
        channel_info = await bot.get_channel_info(guild_id=event.guild_id, channel_id=event.channel_id)
    except ActionFailed:
        channel_info = {}
    try:
        user_info = await bot.get_guild_member_info(guild_id=event.guild_id, user_id=event.operator_id)
    except ActionFailed:
        user_info = {}
    return {
        "guild_id": event.guild_id,
        "guild_name": guild_info.get("guild_name"),
        "channel_id": event.channel_id,
        "channel_name": channel_info.get("channel_name"),
        "user_id": event.operator_id,
        "name": user_info.get("user_name"),
        "nickname": user_info.get("user_displayname"),
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: GuildMemberDecreaseEvent | GuildMemberIncreaseEvent,
):
    try:
        guild_info = await bot.get_guild_info(guild_id=event.guild_id)
    except ActionFailed:
        guild_info = {}
    try:
        user_info = await bot.get_guild_member_info(guild_id=event.guild_id, user_id=event.user_id)
    except ActionFailed:
        user_info = {}
    try:
        operator_info = await bot.get_guild_member_info(guild_id=event.guild_id, user_id=event.operator_id)
    except ActionFailed:
        operator_info = {}
    return {
        "guild_id": event.guild_id,
        "guild_name": guild_info.get("guild_name"),
        "user_id": event.user_id,
        "name": user_info.get("user_name"),
        "nickname": user_info.get("user_displayname"),
        "operator": {
            "guild_id": event.guild_id,
            "guild_name": guild_info.get("guild_name"),
            "user_id": event.operator_id,
            "name": operator_info.get("user_name"),
            "nickname": operator_info.get("user_displayname"),
        },
    }



from nonebot.adapters.satori import Bot
from nonebot.adapters.satori.event import Event
from nonebot.adapters.satori.models import Channel, ChannelType, Friend, Guild
from nonebot.adapters.satori.models import User as SatoriUser

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Role, Scene, SceneType, User

ROLES = {
    "OWNER": ("OWNER", 100),
    "owner": ("OWNER", 100),
    "ADMINISTRATOR": ("ADMINISTRATOR", 10),
    "administrator": ("ADMINISTRATOR", 10),
    "ADMIN": ("ADMINISTRATOR", 10),
    "admin": ("ADMINISTRATOR", 10),
    "CHANNEL_ADMINISTRATOR": ("CHANNEL_ADMINISTRATOR", 8),
    "channel_administrator": ("CHANNEL_ADMINISTRATOR", 8),
    "CHANNEL_ADMIN": ("CHANNEL_ADMINISTRATOR", 8),
    "channel_admin": ("CHANNEL_ADMINISTRATOR", 8),
    "MEMBER": ("MEMBER", 1),
    "member": ("MEMBER", 1),
}


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            nick=data["nickname"],
            avatar=data.get("avatar"),
        )

    def extract_scene(self, data):
        if "scene_id" in data:
            if "parent_id" in data:
                return Scene(
                    id=data["scene_id"],
                    type=data["scene_type"],
                    name=data["scene_name"],
                    parent=Scene(
                        id=data["parent_id"],
                        type=data["parent_type"],
                        name=data.get("parent_name"),
                        avatar=data.get("parent_avatar"),
                    ),
                )
            return Scene(
                id=data["scene_id"],
                type=data["scene_type"],
                name=data["scene_name"],
                avatar=data.get("scene_avatar"),
            )
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
            avatar=data.get("avatar"),
        )

    def extract_member(self, data, user: User | None):
        if "member_name" not in data:
            return None
        if user:
            return Member(
                user=user,
                nick=data["member_name"],
                roles=data.get("member_roles", []),
                joined_at=data["joined_at"],
            )
        return Member(
            User(
                id=data["user_id"],
                name=data["name"],
                nick=data.get("nickname"),
                avatar=data.get("avatar"),
            ),
            nick=data["member_name"],
            roles=data.get("member_roles", []),
            joined_at=data["joined_at"],
        )

    def _pack_user(self, user: SatoriUser | Friend):
        if isinstance(user, Friend):
            data = {
                "user_id": user.user.id,
                "name": user.user.name,
                "nickname": user.nick,
                "avatar": user.user.avatar,
            }
        else:
            data = {
                "user_id": user.id,
                "name": user.name,
                "nickname": user.nick,
                "avatar": user.avatar,
            }
        return self.extract_user(data)

    def _pack_guild(self, bot: Bot, guild: Guild):
        data = {
            "scene_id": guild.id,
            "scene_type": SceneType.GROUP if "guild.plain" in bot._self_info.features else SceneType.GUILD,
            "scene_name": guild.name,
            "scene_avatar": guild.avatar,
        }
        return self.extract_scene(data)

    def _pack_channel(self, bot: Bot, guild: Guild, channel: Channel):
        data = {
            "scene_id": channel.id,
            "scene_type": SceneType.GROUP if "guild.plain" in bot._self_info.features else TYPE_MAPPING[channel.type],
            "scene_name": channel.name,
            "parent_id": guild.id,
            "parent_type": SceneType.GROUP if "guild.plain" in bot._self_info.features else SceneType.GUILD,
            "parent_name": guild.name,
            "parent_avatar": guild.avatar,
        }
        return self.extract_scene(data)

    async def query_user(self, bot: Bot, user_id: str):
        if user_id == bot.self_info.id:
            return self._pack_user(bot.self_info)
        user = await bot.user_get(user_id=user_id)
        return self._pack_user(user)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            user = await bot.user_get(user_id=scene_id)
            data = {
                "scene_id": user.id,
                "scene_type": SceneType.PRIVATE,
                "scene_name": user.name,
                "scene_avatar": user.avatar,
            }
            return self.extract_scene(data)

        if SceneType.GROUP <= scene_type <= SceneType.GUILD:
            guild = await bot.guild_get(guild_id=scene_id)
            return self._pack_guild(bot, guild)

        elif scene_type >= SceneType.CHANNEL_TEXT and parent_scene_id is not None:
            guild = await bot.guild_get(guild_id=parent_scene_id)
            channel = await bot.channel_get(channel_id=scene_id)
            return self._pack_channel(bot, guild, channel)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type in (SceneType.GUILD, SceneType.GROUP):
            member = await bot.guild_member_get(guild_id=parent_scene_id, user_id=user_id)
            if member.user:
                data = {
                    "scene_id": parent_scene_id,
                    "user_id": member.user.id,
                    "name": member.user.name,
                    "nickname": member.user.nick,
                    "avatar": member.avatar or member.user.avatar,
                    "member_name": member.nick,
                    "joined_at": member.joined_at,
                }
                return self.extract_member(data, None)

    async def query_users(self, bot: Bot):
        friends = await bot.friend_list()
        for friend in friends.data:
            yield self._pack_user(friend)
        while friends.next:
            friends = await bot.friend_list(next_token=friends.next)
            for friend in friends.data:
                yield self._pack_user(friend)

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is None or scene_type == SceneType.PRIVATE:
            async for user in self.query_users(bot):
                data = {
                    "scene_id": user.id,
                    "scene_type": SceneType.PRIVATE,
                    "scene_name": user.name,
                    "scene_avatar": user.avatar,
                }
                yield self.extract_scene(data)
            if scene_type == SceneType.PRIVATE:
                return

        guilds = await bot.guild_list()
        for guild in guilds.data:
            if parent_scene_id is None or guild.id == parent_scene_id:
                _guild = self._pack_guild(bot, guild)
                if scene_type is None or scene_type == _guild.type:
                    yield _guild
                if scene_type is not None and scene_type < SceneType.CHANNEL_TEXT:
                    continue
                channels = await bot.channel_list(guild_id=guild.id)
                for channel in channels.data:
                    yield self._pack_channel(bot, guild, channel)
                while channels.next:
                    channels = await bot.channel_list(guild_id=guild.id, next_token=channels.next)
                    for channel in channels.data:
                        yield self._pack_channel(bot, guild, channel)
        while guilds.next:
            guilds = await bot.guild_list(next_token=guilds.next)
            for guild in guilds.data:
                if parent_scene_id is None or guild.id == parent_scene_id:
                    _guild = self._pack_guild(bot, guild)
                    if scene_type is None or scene_type == _guild.type:
                        yield _guild
                    if scene_type is not None and scene_type < SceneType.CHANNEL_TEXT:
                        continue
                    channels = await bot.channel_list(guild_id=guild.id)
                    for channel in channels.data:
                        yield self._pack_channel(bot, guild, channel)
                    while channels.next:
                        channels = await bot.channel_list(guild_id=guild.id, next_token=channels.next)
                        for channel in channels.data:
                            yield self._pack_channel(bot, guild, channel)

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        if scene_type not in (SceneType.GUILD, SceneType.GROUP):
            return
        guild_id = parent_scene_id
        members = await bot.guild_member_list(guild_id=guild_id)
        for member in members.data:
            if not member.user:
                continue
            data = {
                "scene_id": guild_id,
                "user_id": member.user.id,
                "name": member.user.name,
                "nickname": member.user.nick,
                "avatar": member.avatar or member.user.avatar,
                "member_name": member.nick,
                "joined_at": member.joined_at,
            }
            yield self.extract_member(data, None)

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.satori,
            "scope": SupportScope.ensure_satori(bot.platform),
        }


fetcher = InfoFetcher(SupportAdapter.satori)


TYPE_MAPPING = {
    ChannelType.DIRECT: SceneType.PRIVATE,
    ChannelType.TEXT: SceneType.CHANNEL_TEXT,
    ChannelType.VOICE: SceneType.CHANNEL_VOICE,
    ChannelType.CATEGORY: SceneType.CHANNEL_CATEGORY,
}


@fetcher.supply_wildcard
async def _(bot: Bot, event: Event):
    if not event.user:
        user_id = bot.get_self_id()
        name = ""
        nickname = ""
    else:
        user_id = event.user.id
        name = event.user.name
        nickname = event.user.nick
    base = {
        "user_id": user_id,
        "name": name,
        "nickname": nickname,
    }
    if event.user:
        base["avatar"] = event.user.avatar
    if event.guild and event.channel:
        base["scene_id"] = event.channel.id
        base["scene_type"] = TYPE_MAPPING[event.channel.type]
        base["scene_name"] = event.channel.name
        base["parent_id"] = event.guild.id
        base["parent_type"] = SceneType.GUILD
        base["parent_name"] = event.guild.name
        base["parent_avatar"] = event.guild.avatar
        if "guild.plain" in bot._self_info.features or event.guild.id == event.channel.id:
            base["scene_type"] = SceneType.GROUP
            base["parent_type"] = SceneType.GROUP
    elif event.guild:
        base["scene_id"] = event.guild.id
        base["scene_type"] = SceneType.GROUP if "guild.plain" in bot._self_info.features else SceneType.GUILD
        base["scene_name"] = event.guild.name
        base["scene_avatar"] = event.guild.avatar
    elif event.channel:
        base["scene_id"] = event.channel.id
        base["scene_type"] = (
            SceneType.GROUP if "guild.plain" in bot._self_info.features else TYPE_MAPPING[event.channel.type]
        )
        base["scene_name"] = event.channel.name
    if event.member:
        base["member_name"] = event.member.nick
        base["joined_at"] = event.member.joined_at
        if event.member.avatar:
            base["avatar"] = event.member.avatar
        base["member_roles"] = [Role(*ROLES.get(role.id, (role.id, 1)), role.name) for role in event.member.roles]
    if event.operator:
        base["operator"] = {
            "user_id": event.operator.id,
            "name": event.operator.name,
            "nickname": event.operator.nick,
            "member_name": event.operator.nick,
            "avatar": event.operator.avatar,
            "joined_at": None,
        }
    return base



from nonebot.adapters.minecraft import Bot
from nonebot.adapters.minecraft.event import Event, MessageEvent, NoticeEvent

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Scene, SceneType, User


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
        )

    def extract_scene(self, data):
        return Scene(
            id=data["user_id"],
            type=SceneType.PRIVATE,
            name=data["name"],
        )

    def extract_member(self, data, user: User | None):
        return None

    async def query_user(self, bot: Bot, user_id: str):
        return User(user_id, user_id)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        return Scene(id=scene_id, type=SceneType.PRIVATE, name=scene_id)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        raise NotImplementedError

    def query_users(self, bot: Bot):
        raise NotImplementedError

    def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        raise NotImplementedError

    def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        raise NotImplementedError

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.minecraft,
            "scope": SupportScope.minecraft,
        }


fetcher = InfoFetcher(SupportAdapter.minecraft)


@fetcher.supply_wildcard
async def _(bot: Bot, event: Event):
    if isinstance(event, (MessageEvent, NoticeEvent)):
        return {
            "user_id": str(event.player.uuid or event.player.nickname),
            "name": event.player.nickname,
        }
    raise NotImplementedError



from nonebot.adapters.bilibili_live.bot import Bot, WebBot
from nonebot.adapters.bilibili_live.event import (
    AreaRankChangedEvent,
    Event,
    GuardBuyEvent,
    GuardBuyToastEvent,
    LikeEvent,
    MessageEvent,
    PopularRankChangedEvent,
    RoomAdminEntranceEvent,
    RoomAdminRevokeEvent,
    RoomBlockMsgEvent,
    SendGiftEvent,
    _InteractWordEvent,
)
from nonebot.exception import ActionFailed

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, Scene, SceneType, User


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(id=data["user_id"], name=data.get("user_name", data["user_id"]))

    def extract_scene(self, data):
        if "room_id" in data:
            return Scene(
                id=data["room_id"],
                type=SceneType.GROUP,
                name=data.get("room_name", data["room_id"]),
                avatar=data.get("cover", None),
            )
        return Scene(id=data["user_id"], type=SceneType.PRIVATE, name=data.get("user_name", data["user_id"]))

    def extract_member(self, data, user: User | None):
        if user is None:
            user = self.extract_user(data)
        return Member(user, user.name)

    async def query_user(self, bot: Bot, user_id: str):
        return User(user_id, user_id)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        base = {"room_id": scene_id}
        try:
            assert isinstance(bot, WebBot)
            room_info = await bot.get_room_info(int(scene_id))
            base |= {
                "room_name": room_info.title,
                "cover": room_info.user_cover,
            }
        except (AssertionError, NotImplementedError, ActionFailed):
            pass
        return self.extract_scene(base)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        return Member(await self.query_user(bot, user_id), user_id)

    def query_users(self, bot: Bot):
        raise NotImplementedError

    def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        raise NotImplementedError

    def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        raise NotImplementedError

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.bililive,
            "scope": SupportScope.bililive,
        }


fetcher = InfoFetcher(SupportAdapter.bililive)


@fetcher.supply_wildcard
async def _(bot: Bot, event: Event):
    base = {"room_id": event.room_id}
    try:
        assert isinstance(bot, WebBot)
        room_info = await bot.get_room_info(event.room_id)
        base |= {
            "room_name": room_info.title,
            "cover": room_info.user_cover,
        }
    except (AssertionError, NotImplementedError, ActionFailed):
        pass
    if isinstance(event, MessageEvent):
        base |= {
            "user_id": event.get_user_id(),
            "user_name": event.sender.name,
        }
    elif isinstance(event, (_InteractWordEvent, SendGiftEvent, LikeEvent, RoomBlockMsgEvent)):
        base |= {
            "user_id": event.get_user_id(),
            "user_name": event.uname,
        }
    elif isinstance(event, (GuardBuyEvent, GuardBuyToastEvent)):
        base |= {
            "user_id": event.get_user_id(),
            "user_name": event.username,
        }
    elif isinstance(
        event, (PopularRankChangedEvent, AreaRankChangedEvent, RoomAdminEntranceEvent, RoomAdminRevokeEvent)
    ):
        base |= {
            "user_id": event.get_user_id(),
        }
    else:
        raise NotImplementedError
    return base



from abc import ABCMeta, abstractmethod
import asyncio
from collections import defaultdict
from collections.abc import AsyncGenerator, Awaitable, Callable
from types import UnionType
from typing import Any, TypeVar, Union, get_args, get_origin, get_type_hints

from nonebot import get_plugin_config
from nonebot.adapters import Bot, Event

from .config import Config
from .constraint import SupportAdapter
from .model import BasicInfo, Member, Scene, SceneType, Session, User

TE = TypeVar("TE", bound=Event)
TB = TypeVar("TB", bound=Bot)
Supplier = Callable[[TB, TE], Awaitable[dict]]
TSupplier = TypeVar("TSupplier", bound=Supplier)

try:
    conf = get_plugin_config(Config)
except ValueError:
    conf = Config()


class InfoFetcher(metaclass=ABCMeta):
    def __init__(self, adapter: SupportAdapter):
        self.adapter = adapter
        self.endpoint: dict[type[Event], Callable[[Bot, Event], Awaitable[dict]]] = {}
        self.wildcard: Callable[[Bot, Event], Awaitable[dict]] | None = None
        self.session_cache: dict[str, Session] = {}
        self._timertasks = []
        self._user_cache: defaultdict[str, dict[str, User]] = defaultdict(dict)
        self._scene_cache: defaultdict[str, dict[tuple[int, str, str | None], Scene]] = defaultdict(dict)
        self._member_cache: defaultdict[str, dict[tuple[int, str, str], Member]] = defaultdict(dict)

    def clean(self):
        self._user_cache.clear()
        self._scene_cache.clear()
        self._member_cache.clear()
        self.session_cache.clear()
        for task in self._timertasks:
            task.cancel()
        self._timertasks.clear()

    def supply(self, func: TSupplier) -> TSupplier:
        event_type = get_type_hints(func)["event"]
        if get_origin(event_type) in (Union, UnionType):
            for t in get_args(event_type):
                self.endpoint[t] = func  # type: ignore
        else:
            self.endpoint[event_type] = func  # type: ignore
        return func

    def supply_wildcard(self, func: TSupplier) -> TSupplier:
        self.wildcard = func  # type: ignore
        return func

    @abstractmethod
    def extract_user(self, data: dict[str, Any]) -> User:
        pass

    @abstractmethod
    def extract_scene(self, data: dict[str, Any]) -> Scene:
        pass

    @abstractmethod
    def extract_member(self, data: dict[str, Any], user: User | None) -> Member | None:
        pass

    @abstractmethod
    def supply_self(self, bot) -> BasicInfo:
        pass

    def get_session_id(self, event: Event) -> str:
        return event.get_session_id()

    def parse(self, data: dict) -> Session:
        user = self.extract_user(data)
        return Session(
            self_id=data["self_id"],
            adapter=data["adapter"],
            scope=data["scope"],
            user=user,
            scene=self.extract_scene(data),
            member=self.extract_member(data, user),
            operator=self.extract_member(data["operator"], None) if "operator" in data else None,  # type: ignore
        )

    async def fetch(self, bot: Bot, event: Event) -> Session:
        try:
            sess_id = self.get_session_id(event)
        except ValueError:
            pass
        else:
            if sess_id in self.session_cache:
                return self.session_cache[sess_id]
        func = None
        for t in event.__class__.__mro__[:-1]:
            func = self.endpoint.get(t)
            if func:
                break
        base = self.supply_self(bot)
        try:
            if func:
                data = await func(bot, event)
                sess = self.parse({**base, **data})
            elif self.wildcard:
                data = await self.wildcard(bot, event)
                sess = self.parse({**base, **data})
            else:
                raise NotImplementedError(f"Event {type(event)} not supported yet")
        except NotImplementedError:
            raise NotImplementedError(f"Event {type(event)} not supported yet") from None
        if conf.uninfo_cache:
            try:
                sess_id = self.get_session_id(event)
                self.session_cache[sess_id] = sess
                asyncio.get_running_loop().call_later(conf.uninfo_cache_expire, self.session_cache.pop, sess_id, None)
                key1 = sess.user.id
                self._user_cache[bot.self_id][key1] = sess.user
                asyncio.get_running_loop().call_later(
                    conf.uninfo_cache_expire, self._user_cache[bot.self_id].pop, key1, None
                )
                key2 = (sess.scene.type.value, sess.scene.id, sess.scene.parent.id if sess.scene.parent else None)
                self._scene_cache[bot.self_id][key2] = sess.scene
                asyncio.get_running_loop().call_later(
                    conf.uninfo_cache_expire, self._scene_cache[bot.self_id].pop, key2, None
                )
                if sess.member:
                    key3 = (
                        sess.scene.type.value,
                        sess.scene.parent.id if sess.scene.parent else sess.scene.id,
                        sess.member.id,
                    )
                    self._member_cache[bot.self_id][key3] = sess.member
                    asyncio.get_running_loop().call_later(
                        conf.uninfo_cache_expire, self._member_cache[bot.self_id].pop, key3, None
                    )
            except ValueError:
                pass
        return sess

    @abstractmethod
    async def query_user(self, bot: Bot, user_id: str) -> User | None:
        pass

    async def fetch_user(self, bot: Bot, user_id: str) -> User | None:
        if user_id in self._user_cache[bot.self_id]:
            return self._user_cache[bot.self_id][user_id]
        user = await self.query_user(bot, user_id)
        if user and conf.uninfo_cache:
            self._user_cache[bot.self_id][user_id] = user
            asyncio.get_running_loop().call_later(
                conf.uninfo_cache_expire, self._user_cache[bot.self_id].pop, user_id, None
            )
        return user

    @abstractmethod
    async def query_scene(
        self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None
    ) -> Scene | None:
        pass

    async def fetch_scene(
        self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None
    ) -> Scene | None:
        key = (scene_type.value, scene_id, parent_scene_id)
        if key in self._scene_cache[bot.self_id]:
            return self._scene_cache[bot.self_id][key]
        scene = await self.query_scene(bot, scene_type, scene_id, parent_scene_id=parent_scene_id)
        if scene and conf.uninfo_cache:
            self._scene_cache[bot.self_id][key] = scene
            asyncio.get_running_loop().call_later(
                conf.uninfo_cache_expire, self._scene_cache[bot.self_id].pop, key, None
            )
        return scene

    @abstractmethod
    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str) -> Member | None:
        pass

    async def fetch_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str) -> Member | None:
        key = (scene_type.value, parent_scene_id, user_id)
        if key in self._member_cache[bot.self_id]:
            return self._member_cache[bot.self_id][key]
        member = await self.query_member(bot, scene_type, parent_scene_id, user_id)
        if member and conf.uninfo_cache:
            self._member_cache[bot.self_id][key] = member
            asyncio.get_running_loop().call_later(
                conf.uninfo_cache_expire, self._member_cache[bot.self_id].pop, key, None
            )
        return member

    @abstractmethod
    def query_users(self, bot: Bot) -> AsyncGenerator[User, None]:
        pass

    @abstractmethod
    def query_scenes(
        self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None
    ) -> AsyncGenerator[Scene, None]:
        pass

    @abstractmethod
    def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str) -> AsyncGenerator[Member, None]:
        pass



import asyncio
import json

from nonebot import get_bots, require
from nonebot.adapters import Bot
from nonebot.params import Depends

from .model import BasicInfo, Member, Scene, SceneType, Session, User
from .params import UniSession, get_interface

try:
    require("nonebot_plugin_orm")
    from nonebot_plugin_orm import Model, get_session
    from sqlalchemy import JSON, Integer, String, UniqueConstraint, exc, select
    from sqlalchemy.orm import Mapped, mapped_column
except ImportError:
    raise ImportError("You need to install nonebot_plugin_orm to use this module.")


class BotModel(Model):
    __tablename__ = "nonebot_plugin_uninfo_botmodel"
    __table_args__ = (
        UniqueConstraint(
            "self_id",
            "adapter",
            name="nonebot_plugin_uninfo_unique_bot",
        ),
    )

    id: Mapped[int] = mapped_column(primary_key=True)
    self_id: Mapped[str] = mapped_column(String(64))
    adapter: Mapped[str] = mapped_column(String(32))
    scope: Mapped[str] = mapped_column(String(32))

    def get_bot(self) -> Bot | None:
        for bot in list(get_bots().values()):
            if bot.self_id == self.self_id and bot.adapter.get_name() == self.adapter:
                return bot


class SceneModel(Model):
    __tablename__ = "nonebot_plugin_uninfo_scenemodel"
    __table_args__ = (
        UniqueConstraint(
            "bot_persist_id",
            "scene_id",
            "scene_type",
            name="nonebot_plugin_uninfo_unique_scene",
        ),
    )

    id: Mapped[int] = mapped_column(primary_key=True)
    bot_persist_id: Mapped[int] = mapped_column(Integer)
    parent_scene_persist_id: Mapped[int | None] = mapped_column(Integer)
    scene_id: Mapped[str] = mapped_column(String(64))
    scene_type: Mapped[int] = mapped_column(Integer)
    scene_data: Mapped[dict] = mapped_column(JSON)

    async def to_scene(self) -> Scene:
        parent_scene_model = (
            await get_scene_model(self.parent_scene_persist_id) if self.parent_scene_persist_id else None
        )
        return Scene.load(
            {
                **self.scene_data,
                "id": self.scene_id,
                "type": self.scene_type,
                "parent": (
                    {
                        **parent_scene_model.scene_data,
                        "id": parent_scene_model.scene_id,
                        "type": parent_scene_model.scene_type,
                    }
                    if parent_scene_model
                    else None
                ),
            }
        )

    async def query_scene(self) -> Scene | None:
        bot_model = await get_bot_model(self.bot_persist_id)
        if not (bot := bot_model.get_bot()):
            return None
        if not (interface := get_interface(bot)):
            return None

        scene = await self.to_scene()
        return await interface.get_scene(
            SceneType(scene.type), scene.id, parent_scene_id=scene.parent.id if scene.parent else None
        )


class UserModel(Model):
    __tablename__ = "nonebot_plugin_uninfo_usermodel"
    __table_args__ = (
        UniqueConstraint(
            "bot_persist_id",
            "user_id",
            name="nonebot_plugin_uninfo_unique_user",
        ),
    )

    id: Mapped[int] = mapped_column(primary_key=True)
    bot_persist_id: Mapped[int] = mapped_column(Integer)
    user_id: Mapped[str] = mapped_column(String(64))
    user_data: Mapped[dict] = mapped_column(JSON)

    async def to_user(self) -> User:
        return User(**{**self.user_data, "id": self.user_id})

    async def query_user(self) -> User | None:
        bot_model = await get_bot_model(self.bot_persist_id)
        if not (bot := bot_model.get_bot()):
            return None
        if not (interface := get_interface(bot)):
            return None

        return await interface.get_user(self.user_id)


class SessionModel(Model):
    __tablename__ = "nonebot_plugin_uninfo_sessionmodel"
    __table_args__ = (
        UniqueConstraint(
            "bot_persist_id",
            "scene_persist_id",
            "user_persist_id",
            name="nonebot_plugin_uninfo_unique_session",
        ),
    )

    id: Mapped[int] = mapped_column(primary_key=True)
    bot_persist_id: Mapped[int] = mapped_column(Integer)
    scene_persist_id: Mapped[int] = mapped_column(Integer)
    user_persist_id: Mapped[int] = mapped_column(Integer)
    member_data: Mapped[dict | None] = mapped_column(JSON)

    async def to_session(self) -> Session:
        bot_model = await get_bot_model(self.bot_persist_id)
        scene_model = await get_scene_model(self.scene_persist_id)
        scene = await scene_model.to_scene()
        user_model = await get_user_model(self.user_persist_id)
        user = await user_model.to_user()
        member = Member.load(self.member_data) if self.member_data else None

        return Session(
            self_id=bot_model.self_id,
            adapter=bot_model.adapter,
            scope=bot_model.scope,
            scene=scene,
            user=user,
            member=member,
        )

    async def query_session(self) -> Session | None:
        bot_model = await get_bot_model(self.bot_persist_id)
        if not (bot := bot_model.get_bot()):
            return None
        if not (interface := get_interface(bot)):
            return None

        scene_model = await get_scene_model(self.scene_persist_id)
        scene = await scene_model.to_scene()
        if not (
            scene := await interface.get_scene(
                SceneType(scene.type), scene.id, parent_scene_id=scene.parent.id if scene.parent else None
            )
        ):
            return None

        user_model = await get_user_model(self.user_persist_id)
        if not (user := await interface.get_user(user_model.user_id)):
            return None

        member = await interface.get_member(SceneType(scene_model.scene_type), scene_model.scene_id, user_model.user_id)

        return Session(
            self_id=bot_model.self_id,
            adapter=bot_model.adapter,
            scope=bot_model.scope,
            scene=scene,
            user=user,
            member=member,
        )


_insert_mutex: asyncio.Lock | None = None


def _get_insert_mutex():
    # py3.10以下，Lock必须在event_loop内创建
    global _insert_mutex

    if _insert_mutex is None:
        _insert_mutex = asyncio.Lock()
    # elif sys.version_info < (3, 10):
    #     # 还需要判断loop是否与之前创建的一致
    #     # 单测中不同的test，loop也不一样
    #     # 但是nonebot里loop始终是一样的
    #     if getattr(_insert_mutex, "_loop") != asyncio.get_running_loop():
    #         _insert_mutex = asyncio.Lock()

    return _insert_mutex


async def get_bot_persist_id(basic_info: BasicInfo) -> int:
    statement = (
        select(BotModel)
        .where(BotModel.self_id == basic_info["self_id"])
        .where(BotModel.adapter == basic_info["adapter"].value)
    )
    async with get_session() as db_session:
        if bot_model := (await db_session.scalars(statement)).one_or_none():
            bot_model.scope = basic_info["scope"].value
            await db_session.commit()
            await db_session.refresh(bot_model)
            return bot_model.id

    bot_model = BotModel(
        self_id=basic_info["self_id"],
        adapter=basic_info["adapter"].value,
        scope=basic_info["scope"].value,
    )
    async with _get_insert_mutex():
        try:
            async with get_session() as db_session:
                db_session.add(bot_model)
                await db_session.commit()
                await db_session.refresh(bot_model)
                return bot_model.id
        except exc.IntegrityError:
            async with get_session() as db_session:
                return (await db_session.scalars(statement)).one().id


async def get_scene_persist_id(basic_info: BasicInfo, scene: Scene) -> int:
    bot_persist_id = await get_bot_persist_id(basic_info)
    parent_scene_persist_id = await get_scene_persist_id(basic_info, scene.parent) if scene.parent else None
    scene_data = json.loads(scene.dump_json())

    statement = (
        select(SceneModel)
        .where(SceneModel.bot_persist_id == bot_persist_id)
        .where(SceneModel.scene_id == scene.id)
        .where(SceneModel.scene_type == scene.type.value)
    )
    async with get_session() as db_session:
        if scene_model := (await db_session.scalars(statement)).one_or_none():
            scene_model.parent_scene_persist_id = parent_scene_persist_id
            scene_model.scene_data = scene_data
            await db_session.commit()
            await db_session.refresh(scene_model)
            return scene_model.id

    scene_model = SceneModel(
        bot_persist_id=bot_persist_id,
        parent_scene_persist_id=parent_scene_persist_id,
        scene_id=scene.id,
        scene_type=scene.type.value,
        scene_data=scene_data,
    )
    async with _get_insert_mutex():
        try:
            async with get_session() as db_session:
                db_session.add(scene_model)
                await db_session.commit()
                await db_session.refresh(scene_model)
                return scene_model.id
        except exc.IntegrityError:
            async with get_session() as db_session:
                return (await db_session.scalars(statement)).one().id


async def get_user_persist_id(basic_info: BasicInfo, user: User) -> int:
    bot_persist_id = await get_bot_persist_id(basic_info)
    user_data = json.loads(user.dump_json())

    statement = select(UserModel).where(UserModel.bot_persist_id == bot_persist_id).where(UserModel.user_id == user.id)
    async with get_session() as db_session:
        if user_model := (await db_session.scalars(statement)).one_or_none():
            user_model.user_data = user_data
            await db_session.commit()
            await db_session.refresh(user_model)
            return user_model.id

    user_model = UserModel(
        bot_persist_id=bot_persist_id,
        user_id=user.id,
        user_data=user_data,
    )
    async with _get_insert_mutex():
        try:
            async with get_session() as db_session:
                db_session.add(user_model)
                await db_session.commit()
                await db_session.refresh(user_model)
                return user_model.id
        except exc.IntegrityError:
            async with get_session() as db_session:
                return (await db_session.scalars(statement)).one().id


async def get_session_persist_id(session: Session) -> int:
    bot_persist_id = await get_bot_persist_id(session.basic)
    scene_persist_id = await get_scene_persist_id(session.basic, session.scene)
    user_persist_id = await get_user_persist_id(session.basic, session.user)
    member_data = json.loads(session.member.dump_json()) if session.member else None

    statement = (
        select(SessionModel)
        .where(SessionModel.bot_persist_id == bot_persist_id)
        .where(SessionModel.scene_persist_id == scene_persist_id)
        .where(SessionModel.user_persist_id == user_persist_id)
    )
    async with get_session() as db_session:
        if session_model := (await db_session.scalars(statement)).one_or_none():
            session_model.member_data = member_data
            await db_session.commit()
            await db_session.refresh(session_model)
            return session_model.id

    session_model = SessionModel(
        bot_persist_id=bot_persist_id,
        scene_persist_id=scene_persist_id,
        user_persist_id=user_persist_id,
        member_data=member_data,
    )
    async with _get_insert_mutex():
        try:
            async with get_session() as db_session:
                db_session.add(session_model)
                await db_session.commit()
                await db_session.refresh(session_model)
                return session_model.id
        except exc.IntegrityError:
            async with get_session() as db_session:
                return (await db_session.scalars(statement)).one().id


async def get_bot_model(persist_id: int) -> BotModel:
    async with get_session() as db_session:
        return (await db_session.scalars(select(BotModel).where(BotModel.id == persist_id))).one()


async def get_scene_model(persist_id: int) -> SceneModel:
    async with get_session() as db_session:
        return (await db_session.scalars(select(SceneModel).where(SceneModel.id == persist_id))).one()


async def get_user_model(persist_id: int) -> UserModel:
    async with get_session() as db_session:
        return (await db_session.scalars(select(UserModel).where(UserModel.id == persist_id))).one()


async def get_session_model(persist_id: int) -> SessionModel:
    async with get_session() as db_session:
        return (await db_session.scalars(select(SessionModel).where(SessionModel.id == persist_id))).one()


async def get_bot_orm(session: Session = UniSession()):
    return await get_bot_model(await get_bot_persist_id(session.basic))


def BotOrm() -> BotModel:
    return Depends(get_bot_orm)


async def get_scene_orm(session: Session = UniSession()):
    return await get_scene_model(await get_scene_persist_id(session.basic, session.scene))


def SceneOrm() -> SceneModel:
    return Depends(get_scene_orm)


async def get_user_orm(session: Session = UniSession()):
    return await get_user_model(await get_user_persist_id(session.basic, session.user))


def UserOrm() -> UserModel:
    return Depends(get_user_orm)


async def get_session_orm(session: Session = UniSession()):
    return await get_session_model(await get_session_persist_id(session))


def SessionOrm() -> SessionModel:
    return Depends(get_session_orm)



from datetime import datetime, timedelta

from nonebot.adapters.kritor import Bot
from nonebot.adapters.kritor.event import (
    FriendApplyRequest,
    FriendMessage,
    GroupApplyRequest,
    GroupMessage,
    GuildMessage,
    InvitedJoinGroupRequest,
    NearbyMessage,
    StrangerMessage,
    TempMessage,
)
from nonebot.exception import ActionFailed

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, MuteInfo, Role, Scene, SceneType, User

ROLES = {
    "owner": ("OWNER", 100),
    "admin": ("ADMINISTRATOR", 10),
    "member": ("MEMBER", 1),
    "unknown": ("UNKNOWN", 0),
}


class InfoFetcher(BaseInfoFetcher):
    def extract_user(self, data):
        return User(
            id=str(data["user_id"]),
            name=data["name"],
            nick=data["nickname"],
            avatar=data.get("avatar", f"http://q1.qlogo.cn/g?b=qq&nk={data['user_id']}&s=640"),
        )

    def extract_scene(self, data):
        if "group_id" in data:
            return Scene(
                id=str(data["group_id"]),
                type=SceneType.GROUP,
                name=data.get("group_name"),
                avatar=f"https://p.qlogo.cn/gh/{data['group_id']}/{data['group_id']}/",
            )
        if "guild_id" in data:
            if "channel_id" in data:
                return Scene(
                    id=str(data["channel_id"]),
                    type=SceneType.CHANNEL_TEXT,
                    name=data["channel_name"],
                    parent=Scene(
                        id=str(data["guild_id"]),
                        type=SceneType.GUILD,
                        name=data.get("guild_name"),
                    ),
                )
            return Scene(
                id=str(data["guild_id"]),
                type=SceneType.GUILD,
                name=data.get("guild_name"),
            )
        if "parent_group_id" in data:
            return Scene(
                id=str(data["user_id"]),
                type=SceneType.PRIVATE,
                avatar=f"http://q1.qlogo.cn/g?b=qq&nk={data['user_id']}&s=640",
                parent=Scene(
                    id=str(data["parent_group_id"]),
                    type=SceneType.GROUP,
                    name=data.get("parent_group_name"),
                    avatar=f"https://p.qlogo.cn/gh/{data['parent_group_id']}/{data['parent_group_id']}/",
                ),
            )
        return Scene(
            id=str(data["user_id"]),
            type=SceneType.PRIVATE,
            avatar=f"http://q1.qlogo.cn/g?b=qq&nk={data['user_id']}&s=640",
        )

    def extract_member(self, data, user: User | None):
        if "group_id" not in data or "guild_id" not in data:
            return None
        if user:
            return Member(
                user=user,
                nick=data["card"],
                mute=(
                    MuteInfo(muted=True, duration=timedelta(seconds=data["mute_duration"]))
                    if "mute_duration" in data
                    else None
                ),
                joined_at=datetime.fromtimestamp(data["join_time"]) if "join_time" in data else None,
                roles=[Role(*ROLES[data["role"]], data["role"])] if "role" in data else [],
            )
        return Member(
            User(
                id=str(data["user_id"]),
                name=data["name"],
                nick=data.get("nickname"),
                avatar=data.get("avatar", f"https://q2.qlogo.cn/headimg_dl?dst_uin={data['user_id']}&spec=640"),
            ),
            nick=data["card"],
            mute=(
                MuteInfo(muted=True, duration=timedelta(seconds=data["mute_duration"]))
                if "mute_duration" in data
                else None
            ),
            joined_at=datetime.fromtimestamp(data["join_time"]) if "join_time" in data else None,
            roles=[Role(*ROLES[data["role"]], data["role"])] if "role" in data else [],
        )

    async def query_user(self, bot: Bot, user_id: str):
        if bot.self_id == user_id:
            info = await bot.get_current_account()
            data = {"user_id": info.account_uin, "name": info.account_name}
        else:
            try:
                profile = (await bot.get_friend_profile_card(targets=[int(user_id)])).friends_profile_card[0]
            except ActionFailed:
                profile = (await bot.get_stranger_profile_card(targets=[int(user_id)])).strangers_profile_card[0]
            data = {
                "user_id": profile.uin,
                "name": profile.nick,
                "nickname": profile.remark,
            }
        return self.extract_user(data)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.GROUP:
            group = await bot.get_group_info(group=scene_id)
            return self.extract_scene(
                {
                    "group_id": group.group_id,
                    "group_name": group.group_name,
                }
            )

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type == SceneType.GROUP:
            group_id = parent_scene_id

            member_info = await bot.get_group_member_info(group=group_id, target=user_id)
            group_info = await bot.get_group_info(group=group_id)
            data = {
                "group_id": group_id,
                "user_id": member_info.uin,
                "name": member_info.nick,
                "card": member_info.card,
                "mute_duration": member_info.shut_up_time,
                "join_time": member_info.join_time / 1000,
                "role": (
                    "owner"
                    if member_info.uin == group_info.owner
                    else "admin" if member_info.uin in group_info.admins else "member"
                ),
            }
            return self.extract_member(data, None)

        elif scene_type == SceneType.GUILD:
            guild_id = parent_scene_id

            member = await bot.get_guild_member(guild_id=guild_id, tiny_id=int(user_id))
            data = {
                "guild_id": guild_id,
                "user_id": member.member_info.tiny_id,
                "name": member.member_info.nickname,
                "card": member.member_info.nickname,
                "join_time": member.member_info.join_time / 1000,
                "role": member.member_info.roles_info[0].role_name.lower(),
                "avatar": member.member_info.avatar_url,
            }
            return self.extract_member(data, None)

    async def query_users(self, bot: Bot):
        friends = await bot.get_friend_list()
        for friend in friends:
            data = {
                "user_id": friend.uin,
                "name": friend.nick,
                "nickname": friend.remark,
            }
            yield self.extract_user(data)

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is None or scene_type == SceneType.PRIVATE:
            async for user in self.query_users(bot):
                yield self.extract_scene(
                    {
                        "user_id": user.id,
                        "name": user.name,
                        "avatar": user.avatar,
                    }
                )
            if scene_type == SceneType.PRIVATE:
                return

        if scene_type is None or scene_type == SceneType.GROUP:
            groups = await bot.get_group_list()
            for group in groups:
                yield self.extract_scene(
                    {
                        "group_id": group.group_id,
                        "group_name": group.group_name,
                    }
                )
            if scene_type == SceneType.GROUP:
                return

        guilds = await bot.get_guild_list()
        for guild in guilds:
            if parent_scene_id is None or guild.guild_id == parent_scene_id:
                data = {
                    "guild_id": guild.guild_id,
                    "guild_name": guild.guild_name,
                }
                if scene_type is None or scene_type == SceneType.GUILD:
                    yield self.extract_scene(data)
                if scene_type == SceneType.GUILD:
                    continue
                channels = await bot.get_guild_channel_list(guild_id=guild.guild_id)
                for channel in channels:
                    data = {
                        "guild_id": guild.guild_id,
                        "guild_name": guild.guild_name,
                        "channel_id": channel.channel_id,
                        "channel_name": channel.channel_name,
                    }
                    yield self.extract_scene(data)

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        if scene_type == SceneType.GROUP:
            group_id = parent_scene_id
            group_members = await bot.get_group_member_list(group=group_id)
            group_info = await bot.get_group_info(group=group_id)
            admins = group_info.admins
            owner = [group_info.owner]
            for member in group_members:
                data = {
                    "group_id": group_id,
                    "user_id": member.uin,
                    "name": member.nick,
                    "card": member.card,
                    "mute_duration": member.shut_up_time,
                    "join_time": member.join_time / 1000,
                    "role": "owner" if member.uin in owner else "admin" if member.uin in admins else "member",
                }
                yield self.extract_member(data, None)

        elif scene_type == SceneType.GUILD:
            guild_id = parent_scene_id
            guild_members = await bot.get_guild_member_list(guild_id=guild_id)
            for member in guild_members.members_info:
                try:
                    member_prof = await bot.get_guild_member(guild_id=guild_id, tiny_id=member.tiny_id)
                    avatar = member_prof.member_info.avatar_url
                except ActionFailed:
                    avatar = None
                data = {
                    "guild_id": guild_id,
                    "user_id": member.tiny_id,
                    "name": member.nickname,
                    "card": member.nickname,
                    "join_time": member.join_time / 1000,
                    "role": member.role_name.lower(),
                    "avatar": avatar,
                }
                yield self.extract_member(data, None)
            while guild_members.next_token and not guild_members.finished:
                guild_members = await bot.get_guild_member_list(guild_id=guild_id, next_token=guild_members.next_token)
                for member in guild_members.members_info:
                    try:
                        member_prof = await bot.get_guild_member(guild_id=guild_id, tiny_id=member.tiny_id)
                        avatar = member_prof.member_info.avatar_url
                    except ActionFailed:
                        avatar = None
                    data = {
                        "guild_id": guild_id,
                        "user_id": member.tiny_id,
                        "name": member.nickname,
                        "card": member.nickname,
                        "join_time": member.join_time / 1000,
                        "role": member.role_name.lower(),
                        "avatar": avatar,
                    }
                    yield self.extract_member(data, None)

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.kritor,
            "scope": SupportScope.qq_client,
        }


fetcher = InfoFetcher(SupportAdapter.kritor)


@fetcher.supply
async def _(bot: Bot, event: FriendMessage):
    try:
        user_info = await bot.get_friend_profile_card(targets=[event.sender.uin])
        remark = user_info.friends_profile_card[0].remark
    except ActionFailed:
        user_info = {}
        remark = None
    return {
        "user_id": event.sender.uin,
        "name": event.sender.nick,
        "nickname": remark,
    }


@fetcher.supply
async def _(bot: Bot, event: GroupMessage):
    try:
        group_info = await bot.get_group_info(group=event.sender.group_id)
        member_info = await bot.get_group_member_info(group=event.sender.group_id, target=event.sender.uin)
        nick = member_info.nick
        card = member_info.card
        group_name = group_info.group_name
        group_admins = group_info.admins
        group_owner = group_info.owner
        extra = {
            "mute_duration": member_info.shut_up_time,
            "join_time": member_info.join_time / 1000,
            "role": (
                "owner"
                if event.sender.uin == group_owner
                else "admin" if event.sender.uin in group_admins else "member"
            ),
        }
    except ActionFailed:
        nick = card = event.sender.nick
        group_name = ""
        extra = {}
    return {
        "user_id": event.sender.uin,
        "group_id": event.sender.group_id,
        "name": nick,
        "card": card,
        "group_name": group_name,
        **extra,
    }


@fetcher.supply
async def _(bot: Bot, event: StrangerMessage | NearbyMessage):
    try:
        user_info = await bot.get_stranger_profile_card(targets=[event.sender.uin])
        remark = user_info.strangers_profile_card[0].remark
    except ActionFailed:
        remark = None
    return {
        "user_id": event.sender.uin,
        "name": event.sender.nick,
        "nickname": remark,
    }


@fetcher.supply
async def _(bot: Bot, event: TempMessage):
    try:
        group_info = await bot.get_group_info(group=event.sender.group_id)
        member_info = await bot.get_group_member_info(group=event.sender.group_id, target=event.sender.uin)
        nick = member_info.nick
        card = member_info.card
        group_name = group_info.group_name
        group_admins = group_info.admins
        group_owner = group_info.owner
        extra = {
            "mute_duration": member_info.shut_up_time,
            "join_time": member_info.join_time / 1000,
            "role": (
                "owner"
                if event.sender.uin == group_owner
                else "admin" if event.sender.uin in group_admins else "member"
            ),
        }
    except ActionFailed:
        nick = card = event.sender.nick
        group_name = ""
        extra = {}
    return {
        "user_id": event.sender.uin,
        "parent_group_id": event.sender.group_id,
        "name": nick,
        "card": card,
        "parent_group_name": group_name,
        **extra,
    }


@fetcher.supply
async def _(bot: Bot, event: GuildMessage):
    try:
        guilds = await bot.get_guild_list()
        guild_info = next(guild for guild in guilds if guild.guild_id == event.sender.guild_id)
        guild_name = guild_info.guild_name
        channels = await bot.get_guild_channel_list(guild_id=event.sender.guild_id)
        channel_info = next(channel for channel in channels if channel.channel_id == event.sender.channel_id)
        channel_name = channel_info.channel_name
    except (ActionFailed, StopIteration):
        guild_name = ""
        channel_name = ""
    try:
        member = await bot.get_guild_member(guild_id=event.sender.guild_id, tiny_id=int(event.sender.tiny_id))
        card = member.member_info.nickname
        extra = {
            "avatar": member.member_info.avatar_url,
            "join_time": member.member_info.join_time / 1000,
        }
    except (ActionFailed, StopIteration):
        card = event.sender.nick
        extra = {}
    return {
        "user_id": event.sender.tiny_id,
        "name": event.sender.nick,
        "card": card,
        "guild_id": event.sender.guild_id,
        "guild_name": guild_name,
        "channel_id": event.sender.channel_id,
        "channel_name": channel_name,
        "role": event.sender.role.name.lower(),
        **extra,
    }


@fetcher.supply
async def _(bot: Bot, event: GroupApplyRequest):
    try:
        user_info = await bot.get_stranger_profile_card(targets=[event.applier_uin])
        nick = user_info.strangers_profile_card[0].nick
        remark = user_info.strangers_profile_card[0].remark
    except ActionFailed:
        nick = ""
        remark = None
    try:
        group_info = await bot.get_group_info(group=event.group_id)
        group_name = group_info.group_name
    except ActionFailed:
        group_name = ""
    return {
        "user_id": event.applier_uin,
        "name": nick,
        "nickname": remark,
        "group_id": event.group_id,
        "group_name": group_name,
    }


@fetcher.supply
async def _(bot: Bot, event: FriendApplyRequest):
    try:
        user_info = await bot.get_stranger_profile_card(targets=[event.applier_uin])
        nick = user_info.strangers_profile_card[0].nick
        remark = user_info.strangers_profile_card[0].remark
    except ActionFailed:
        nick = ""
        remark = None
    return {
        "user_id": event.applier_uin,
        "name": nick,
        "nickname": remark,
    }


@fetcher.supply
async def _(bot: Bot, event: InvitedJoinGroupRequest):
    try:
        user_info = await bot.get_stranger_profile_card(targets=[event.inviter_uin])
        nick = user_info.strangers_profile_card[0].nick
        remark = user_info.strangers_profile_card[0].remark
    except ActionFailed:
        nick = ""
        remark = None
    try:
        group_info = await bot.get_group_info(group=event.group_id)
        group_name = group_info.group_name
    except ActionFailed:
        group_name = ""
    return {
        "user_id": event.inviter_uin,
        "name": nick,
        "nickname": remark,
        "group_id": event.group_id,
        "group_name": group_name,
    }



from datetime import datetime, timedelta

from nonebot.adapters.onebot.v11 import Bot
from nonebot.adapters.onebot.v11.event import (
    FriendAddNoticeEvent,
    FriendRecallNoticeEvent,
    FriendRequestEvent,
    GroupAdminNoticeEvent,
    GroupBanNoticeEvent,
    GroupDecreaseNoticeEvent,
    GroupIncreaseNoticeEvent,
    GroupMessageEvent,
    GroupRecallNoticeEvent,
    GroupRequestEvent,
    GroupUploadNoticeEvent,
    HonorNotifyEvent,
    PokeNotifyEvent,
    PrivateMessageEvent,
)
from nonebot.exception import ActionFailed
from nonebot.internal.adapter import Event

from nonebot_plugin_uninfo.constraint import SupportAdapter, SupportScope
from nonebot_plugin_uninfo.fetch import BasicInfo
from nonebot_plugin_uninfo.fetch import InfoFetcher as BaseInfoFetcher
from nonebot_plugin_uninfo.model import Member, MuteInfo, Role, Scene, SceneType, User

ROLES = {
    "owner": ("OWNER", 100),
    "admin": ("ADMINISTRATOR", 10),
    "member": ("MEMBER", 1),
}


class InfoFetcher(BaseInfoFetcher):
    def get_session_id(self, event: Event) -> str:
        if isinstance(event, PokeNotifyEvent):
            return f"{event.get_session_id()}_{event.target_id}"
        if isinstance(
            event,
            (
                GroupDecreaseNoticeEvent,
                GroupIncreaseNoticeEvent,
                GroupRecallNoticeEvent,
                GroupBanNoticeEvent,
            ),
        ):
            return f"{event.get_session_id()}_{event.operator_id}"
        return event.get_session_id()

    def extract_user(self, data):
        return User(
            id=data["user_id"],
            name=data["name"],
            nick=data["nickname"],
            avatar=f"http://q1.qlogo.cn/g?b=qq&nk={data['user_id']}&s=640",
            gender=data.get("gender", "unknown"),
        )

    def extract_scene(self, data):
        if "group_id" not in data:
            return Scene(
                id=data["user_id"],
                type=SceneType.PRIVATE,
                name=data["name"],
                avatar=f"http://q1.qlogo.cn/g?b=qq&nk={data['user_id']}&s=640",
            )
        return Scene(
            id=data["group_id"],
            type=SceneType.GROUP,
            name=data["group_name"],
            avatar=f"https://p.qlogo.cn/gh/{data['group_id']}/{data['group_id']}/",
        )

    def extract_member(self, data, user: User | None):
        if "group_id" not in data:
            return None
        if user:
            return Member(
                user=user,
                nick=data["card"],
                roles=[Role(*ROLES[_role], name=_role)] if (_role := data.get("role")) else [],
                joined_at=datetime.fromtimestamp(data["join_time"]) if data["join_time"] else None,
                mute=(
                    MuteInfo(muted=True, duration=timedelta(seconds=data["mute_duration"]))
                    if "mute_duration" in data
                    else None
                ),
            )
        return Member(
            User(
                id=data["user_id"],
                name=data["name"],
                nick=data.get("nickname"),
                avatar=f"https://q2.qlogo.cn/headimg_dl?dst_uin={data['user_id']}&spec=640",
            ),
            nick=data["card"],
            roles=[Role(*ROLES[_role], name=_role)] if (_role := data.get("role")) else [],
            joined_at=datetime.fromtimestamp(data["join_time"]) if data["join_time"] else None,
            mute=(
                MuteInfo(muted=True, duration=timedelta(seconds=data["mute_duration"]))
                if "mute_duration" in data
                else None
            ),
        )

    async def query_user(self, bot: Bot, user_id: str):
        if user_id == bot.self_id:
            info = await bot.get_login_info()
        else:
            info = await bot.get_stranger_info(user_id=int(user_id))
        data = {
            "user_id": str(info["user_id"]),
            "name": info["nickname"],
            "nickname": info["nickname"],
            "gender": info.get("sex"),
        }
        return self.extract_user(data)

    async def query_scene(self, bot: Bot, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None):
        if scene_type == SceneType.PRIVATE:
            if user := (await self.query_user(bot, scene_id)):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "avatar": user.avatar,
                }
                return self.extract_scene(data)

        elif scene_type == SceneType.GROUP:
            group = await bot.get_group_info(group_id=int(scene_id))
            data = {
                "group_id": str(group["group_id"]),
                "group_name": group["group_name"],
            }
            return self.extract_scene(data)

    async def query_member(self, bot: Bot, scene_type: SceneType, parent_scene_id: str, user_id: str):
        if scene_type != SceneType.GROUP:
            return
        group_id = parent_scene_id

        member = await bot.get_group_member_info(group_id=int(group_id), user_id=int(user_id))
        data = {
            "group_id": group_id,
            "user_id": str(member["user_id"]),
            "name": member["nickname"],
            "card": member["card"],
            "role": member["role"],
            "join_time": member.get("join_time"),
            "gender": member["sex"],
        }
        return self.extract_member(data, None)

    async def query_users(self, bot: Bot):
        friends = await bot.get_friend_list()
        for friend in friends:
            data = {
                "user_id": str(friend["user_id"]),
                "name": friend["nickname"],
                "nickname": friend["remark"],
            }
            yield self.extract_user(data)

    async def query_scenes(self, bot: Bot, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        if scene_type is None or scene_type == SceneType.PRIVATE:
            async for user in self.query_users(bot):
                data = {
                    "user_id": user.id,
                    "name": user.name,
                    "avatar": user.avatar,
                }
                yield self.extract_scene(data)

        if scene_type is None or scene_type == SceneType.GROUP:
            groups = await bot.get_group_list()
            for group in groups:
                data = {
                    "group_id": str(group["group_id"]),
                    "group_name": group["group_name"],
                }
                yield self.extract_scene(data)

    async def query_members(self, bot: Bot, scene_type: SceneType, parent_scene_id: str):
        if scene_type != SceneType.GROUP:
            return
        group_id = parent_scene_id

        members = await bot.get_group_member_list(group_id=int(group_id))
        for member in members:
            data = {
                "group_id": group_id,
                "user_id": str(member["user_id"]),
                "name": member["nickname"],
                "card": member["card"],
                "role": member["role"],
                "join_time": member.get("join_time"),
                "gender": member["sex"],
            }
            yield self.extract_member(data, None)

    def supply_self(self, bot: Bot) -> BasicInfo:
        return {
            "self_id": str(bot.self_id),
            "adapter": SupportAdapter.onebot11,
            "scope": SupportScope.qq_client,
        }


fetcher = InfoFetcher(SupportAdapter.onebot11)


@fetcher.supply
async def _(bot: Bot, event: PrivateMessageEvent):
    return {
        "user_id": str(event.user_id),
        "name": event.sender.nickname,
        "nickname": event.sender.card,
        "gender": event.sender.sex or "unknown",
    }


@fetcher.supply
async def _(bot: Bot, event: FriendAddNoticeEvent | FriendRecallNoticeEvent | FriendRequestEvent):
    async for friend in fetcher.query_users(bot):
        if friend.id == str(event.user_id):
            friend_info = {
                "nickname": friend.name,
                "remark": friend.nick,
            }
            break
    else:
        try:
            friend_info = await bot.get_stranger_info(user_id=event.user_id)
        except ActionFailed:
            friend_info = {}
    return {
        "user_id": str(event.user_id),
        "name": friend_info.get("nickname"),
        "nickname": friend_info.get("remark"),
        "gender": friend_info.get("sex", "unknown"),
    }


@fetcher.supply
async def _(bot: Bot, event: GroupMessageEvent):
    try:
        group_info = await bot.get_group_info(group_id=event.group_id)
    except ActionFailed:
        group_info = {}
    try:
        member_info = await bot.get_group_member_info(group_id=event.group_id, user_id=event.user_id, no_cache=True)
    except ActionFailed:
        member_info = {}
    return {
        "group_id": str(event.group_id),
        "group_name": group_info.get("group_name"),
        "user_id": str(event.user_id),
        "name": event.sender.nickname,
        "nickname": event.sender.card,
        "card": member_info.get("card"),
        "role": event.sender.role,
        "join_time": member_info.get("join_time"),
        "gender": member_info.get("sex", "unknown"),
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: GroupUploadNoticeEvent | GroupAdminNoticeEvent | GroupRequestEvent | HonorNotifyEvent,
):
    try:
        group_info = await bot.get_group_info(group_id=event.group_id)
    except ActionFailed:
        group_info = {}
    try:
        member_info = await bot.get_group_member_info(group_id=event.group_id, user_id=event.user_id, no_cache=True)
    except ActionFailed:
        member_info = {}
    return {
        "group_id": str(event.group_id),
        "group_name": group_info.get("group_name"),
        "user_id": str(event.user_id),
        "name": member_info.get("nickname"),
        "nickname": member_info.get("card"),
        "card": member_info.get("card"),
        "role": member_info.get("role"),
        "join_time": member_info.get("join_time"),
        "gender": member_info.get("sex", "unknown"),
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: PokeNotifyEvent,
):
    if not event.group_id:
        async for friend in fetcher.query_users(bot):
            if friend.id == str(event.user_id):
                friend_info = {
                    "nickname": friend.name,
                    "remark": friend.nick,
                }
                break
        else:
            try:
                friend_info = await bot.get_stranger_info(user_id=event.user_id)
            except ActionFailed:
                friend_info = {}
        return {
            "user_id": str(event.user_id),
            "name": friend_info.get("nickname"),
            "nickname": friend_info.get("remark"),
            "gender": friend_info.get("sex", "unknown"),
            "operator": {
                "user_id": str(event.user_id),
                "name": friend_info.get("nickname"),
                "nickname": friend_info.get("remark"),
                "gender": friend_info.get("sex", "unknown"),
            },
        }
    try:
        group_info = await bot.get_group_info(group_id=event.group_id)
    except ActionFailed:
        group_info = {}
    try:
        operator_info = await bot.get_group_member_info(group_id=event.group_id, user_id=event.user_id, no_cache=True)
    except ActionFailed:
        operator_info = {}
    try:
        member_info = await bot.get_group_member_info(group_id=event.group_id, user_id=event.target_id, no_cache=True)
    except ActionFailed:
        member_info = {}
    return {
        "group_id": str(event.group_id),
        "group_name": group_info.get("group_name"),
        "user_id": str(event.target_id),
        "name": member_info.get("nickname"),
        "nickname": member_info.get("card"),
        "card": member_info.get("card"),
        "role": member_info.get("role"),
        "join_time": member_info.get("join_time"),
        "gender": member_info.get("sex", "unknown"),
        "operator": {
            "group_id": str(event.group_id),
            "user_id": str(event.user_id),
            "name": operator_info.get("nickname"),
            "nickname": operator_info.get("card"),
            "card": operator_info.get("card"),
            "role": operator_info.get("role"),
            "join_time": operator_info.get("join_time"),
            "gender": operator_info.get("sex", "unknown"),
        },
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: GroupDecreaseNoticeEvent | GroupIncreaseNoticeEvent | GroupRecallNoticeEvent,
):
    try:
        group_info = await bot.get_group_info(group_id=event.group_id)
    except ActionFailed:
        group_info = {}
    try:
        member_info = await bot.get_group_member_info(group_id=event.group_id, user_id=event.user_id, no_cache=True)
    except ActionFailed:
        member_info = {}
    try:
        operator_info = await bot.get_group_member_info(
            group_id=event.group_id, user_id=event.operator_id, no_cache=True
        )
    except ActionFailed:
        operator_info = {}
    return {
        "group_id": str(event.group_id),
        "group_name": group_info.get("group_name"),
        "user_id": str(event.user_id),
        "name": member_info.get("nickname"),
        "nickname": member_info.get("card"),
        "card": member_info.get("card"),
        "role": member_info.get("role"),
        "join_time": member_info.get("join_time"),
        "gender": member_info.get("sex", "unknown"),
        "operator": {
            "group_id": str(event.group_id),
            "user_id": str(event.operator_id),
            "name": operator_info.get("nickname"),
            "nickname": operator_info.get("card"),
            "card": operator_info.get("card"),
            "role": operator_info.get("role"),
            "join_time": operator_info.get("join_time"),
            "gender": operator_info.get("sex", "unknown"),
        },
    }


@fetcher.supply
async def _(
    bot: Bot,
    event: GroupBanNoticeEvent,
):
    try:
        group_info = await bot.get_group_info(group_id=event.group_id)
    except ActionFailed:
        group_info = {}
    try:
        member_info = await bot.get_group_member_info(group_id=event.group_id, user_id=event.user_id, no_cache=True)
    except ActionFailed:
        member_info = {}
    try:
        operator_info = await bot.get_group_member_info(
            group_id=event.group_id, user_id=event.operator_id, no_cache=True
        )
    except ActionFailed:
        operator_info = {}
    return {
        "group_id": str(event.group_id),
        "group_name": group_info.get("group_name"),
        "user_id": str(event.user_id),
        "name": member_info.get("nickname"),
        "nickname": member_info.get("card"),
        "card": member_info.get("card"),
        "role": member_info.get("role"),
        "join_time": member_info.get("join_time"),
        "gender": member_info.get("sex", "unknown"),
        "mute_duration": event.duration,
        "operator": {
            "group_id": str(event.group_id),
            "user_id": str(event.operator_id),
            "name": operator_info.get("nickname"),
            "nickname": operator_info.get("card"),
            "card": operator_info.get("card"),
            "role": operator_info.get("role"),
            "join_time": operator_info.get("join_time"),
            "gender": operator_info.get("sex", "unknown"),
        },
    }



from typing import Annotated

from nonebot.adapters import Bot
from nonebot.params import Depends

from .adapters import INFO_FETCHER_MAPPING, alter_get_fetcher
from .fetch import InfoFetcher
from .model import Member, Scene, SceneType, Session, User


async def get_session(bot: Bot, event):
    adapter = bot.adapter.get_name()
    fetcher = INFO_FETCHER_MAPPING.get(adapter)
    if not fetcher:
        fetcher = alter_get_fetcher(adapter)
    if fetcher:
        try:
            return await fetcher.fetch(bot, event)
        except NotImplementedError:
            pass
    return None


def UniSession() -> Session:
    return Depends(get_session)


Uninfo = Annotated[Session, UniSession()]


class Interface:
    def __init__(self, bot: Bot, fetcher: InfoFetcher):
        self.bot = bot
        self.fetcher = fetcher

    def basic_info(self):
        return self.fetcher.supply_self(self.bot)

    async def get_user(self, user_id: str) -> User | None:
        """根据用户id获取用户信息

        若适配器不支持该方法，则
            1. 遍历所有用户信息，直到找到对应的用户
            2. 返回空的用户信息
            3. 返回 None

        Args:
            user_id (str): 需要获取的用户id
        """
        try:
            return await self.fetcher.fetch_user(self.bot, user_id)
        except NotImplementedError:
            pass

        async for user in self.iter_users():
            if user.id == user_id:
                return user

    async def get_scene(
        self, scene_type: SceneType, scene_id: str, *, parent_scene_id: str | None = None
    ) -> Scene | None:
        """根据场景类型和场景id获取场景信息

        若适配器不支持该方法，则
            1. 遍历所有场景信息，直到找到对应的场景
            2. 返回空的场景信息
            3. 返回 None

        Args:
            scene_type (SceneType): 需要获取的场景类型 (如群组、频道等)
            scene_id (str): 场景id (如群号、子频道id等)
            parent_scene_id (str, optional): 父场景id. Defaults to None.
        """
        try:
            return await self.fetcher.fetch_scene(self.bot, scene_type, scene_id, parent_scene_id=parent_scene_id)
        except NotImplementedError:
            pass

        async for scene in self.iter_scenes(scene_type, parent_scene_id=parent_scene_id):
            if scene.type == scene_type and scene.id == scene_id:
                return scene

    async def get_member(self, scene_type: SceneType, scene_id: str, user_id: str) -> Member | None:
        """根据场景类型、场景id和用户id获取成员信息

        若适配器不支持该方法，则
            1. 遍历所有成员信息，直到找到对应的成员
            2. 返回空的成员信息
            3. 返回 None

        Args:
            scene_type (SceneType): 成员所属的场景类型 (如群组、频道等)
            scene_id (str): 成员所属的场景id (如群号、频道id等)
            user_id (str): 成员的用户id
        """
        try:
            return await self.fetcher.fetch_member(self.bot, scene_type, scene_id, user_id)
        except NotImplementedError:
            pass

        async for member in self.iter_members(scene_type, scene_id):
            if member.user.id == user_id:
                return member

    async def get_users(self) -> list[User]:
        """获取所有用户信息"""
        ans = []
        async for user in self.iter_users():
            ans.append(user)
        return ans

    async def get_scenes(
        self, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None
    ) -> list[Scene]:
        """获取所有场景信息

        Args:
            scene_type (SceneType, optional): 场景类型. Defaults to None.
            parent_scene_id (str, optional): 父场景id. Defaults to None.
        """
        ans = []
        async for scene in self.iter_scenes(scene_type, parent_scene_id=parent_scene_id):
            ans.append(scene)
        return ans

    async def get_members(self, scene_type: SceneType, scene_id: str) -> list[Member]:
        """获取所有成员信息

        Args:
            scene_type (SceneType): 成员所属的场景类型 (如群组、频道等)
            scene_id (str): 成员所属的场景id (如群号、频道id等)
        """
        ans = []
        async for member in self.iter_members(scene_type, scene_id):
            ans.append(member)
        return ans

    async def iter_users(self):
        """查询所有用户信息的迭代方法"""
        try:
            async for user in self.fetcher.query_users(self.bot):
                yield user
        except NotImplementedError:
            return

    async def iter_scenes(self, scene_type: SceneType | None = None, *, parent_scene_id: str | None = None):
        """查询所有场景信息的迭代方法

        Args:
            scene_type (SceneType, optional): 场景类型. Defaults to None.
            parent_scene_id (str, optional): 父场景id. Defaults to None.
        """
        try:
            async for scene in self.fetcher.query_scenes(self.bot, scene_type, parent_scene_id=parent_scene_id):
                yield scene
        except NotImplementedError:
            return

    async def iter_members(self, scene_type: SceneType, scene_id: str):
        """查询所有成员信息的迭代方法

        Args:
            scene_type (SceneType): 成员所属的场景类型 (如群组、频道等)
            scene_id (str): 成员所属的场景id (如群号、频道id等)
        """
        try:
            async for member in self.fetcher.query_members(self.bot, scene_type, scene_id):
                yield member
        except NotImplementedError:
            return


def get_interface(bot: Bot):
    adapter = bot.adapter.get_name()
    fetcher = INFO_FETCHER_MAPPING.get(adapter)
    if fetcher:
        return Interface(bot, fetcher)
    return None


def QueryInterface() -> Interface:
    return Depends(get_interface)


QryItrface = Annotated[Interface, QueryInterface()]
